# -*- coding: utf-8 -*-

"""
Created on 2016-06-15
:author: Oshane Bailey (b4.oshany@gmail.com)
"""

import colander
import deform
from kotti.views.edit import ContentSchema
from kotti.views.form import AddFormView
from kotti.views.form import EditFormView
from pyramid.view import view_config

from kotti_site_settings import _
from kotti_site_settings.resources import AppSettingsPage


class AppSettingsPageSchema(ContentSchema):
    """ Schema for CustomContent. """
    title = colander.SchemaNode(
        colander.String(),
        title=_(u'Settings for:'),
    )


@view_config(name=AppSettingsPage.type_info.add_view,
             permission=AppSettingsPage.type_info.add_permission,
             renderer='kotti:templates/edit/node.pt')
class AppSettingsPageAddForm(AddFormView):
    """ Form to add a new instance of CustomContent. """

    schema_factory = AppSettingsPageSchema
    add = AppSettingsPage
    item_type = _(u"System Settings")

    def save_success(self, appstruct):
        title = appstruct["title"]
        if not title.lower().endswith("settings"):
            appstruct["title"] = "{} Settings".format(title)
        appstruct["editor_name"] = "{}".format(self.request.user.name)
        return super(AppSettingsPageAddForm, self).save_success(appstruct)


@view_config(name='edit', context=AppSettingsPage, permission='edit',
             renderer='kotti:templates/edit/node.pt')
class AppSettingsPageEditForm(EditFormView):
    """ Form to edit existing CustomContent objects. """

    schema_factory = AppSettingsPageSchema

    def save_success(self, appstruct):
        appstruct["editor_name"] = "{}".format(self.request.user.name)
        return super(AppSettingsPageEditForm, self).save_success(appstruct)



# -*- coding: utf-8 -*-

"""
Created on 2016-06-15
:author: Oshane Bailey (b4.oshany@gmail.com)
"""

from kotti import Base
from kotti.interfaces import IDefaultWorkflow
from kotti.resources import Content
from sqlalchemy import (
    Column,
    String,
    Unicode,
    Integer,
    ForeignKey
)
from sqlalchemy.orm import relationship
from sqlalchemy.ext.declarative import declared_attr
from zope.interface import implements

from kotti_site_settings import _


class AppSettingsPage(Content):
    """Survey Content type."""

    tablename = 'setting_pages'

    @declared_attr
    def __tablename__(cls):
        return cls.tablename

    implements(IDefaultWorkflow)

    id = Column(Integer, ForeignKey('contents.id'),
                primary_key=True, unique=True)
    editor_name = Column(
        Unicode,
        ForeignKey('principals.name', onupdate="CASCADE", ondelete="CASCADE"),
        primary_key=True
    )

    @declared_attr
    def editor(cls):
        return relationship(
            "kotti.security.Principal",
            primaryjoin=("kotti.security.Principal.name=="
                         "AppSettingsPage.editor_name"))

    type_info = Content.type_info.copy(
        name=u'System Settings',
        title=_(u'System Settings'),
        add_view=u'add_settings',
        addable_to=[u'Document'],
        add_permission="admin"
    )


class AppSettings(Base):

    tablename = 'site_settings'

    @declared_attr
    def __tablename__(cls):
        return cls.tablename

    id = Column(Unicode(100), primary_key=True, nullable=False)
    settings_id = Column(
        Integer,
        ForeignKey('setting_pages.id', onupdate="CASCADE", ondelete="CASCADE"),
        primary_key=True)
    option = Column(String, nullable=False)
    value = Column(String, default="")



# -*- coding: utf-8 -*-

"""
Created on 2016-06-15
:author: Oshane Bailey (b4.oshany@gmail.com)
"""

from pyramid.view import view_config
from pyramid.view import view_defaults
from pyramid import httpexceptions as httpexc

from kotti import DBSession
from kotti_site_settings import _, util
from kotti_site_settings.resources import AppSettings, AppSettingsPage
from kotti_site_settings.fanstatic import css_and_js
from kotti_site_settings.views import BaseView


class BaseSettingViews(BaseView):
    @view_config(name='site-settings', permission='admin', root_only=True,
                 renderer='kotti_site_settings:templates/settings.pt')
    def default_view(self):
        settings = AppSettingsPage.query.all()
        return {
            "settings": settings
        }


@view_defaults(context=AppSettingsPage, permission='admin')
class AppSettingsViews(BaseView):
    """ Views for :class:`kotti_site_settings.resources.CustomContent` """

    @view_config(name='view', permission='admin',
                 renderer='kotti_site_settings:templates/options.pt')
    def default_view(self):
        """ Default view for :class:`kotti_site_settings.resources.CustomContent`

        :result: Dictionary needed to render the template.
        :rtype: dict
        """
        app_settings = AppSettings.query.filter(
            AppSettings.settings_id == self.context.id
        ).all()
        return {
            'app_settings': app_settings
        }

    @view_config(name='update-settings', permission='admin',
                 request_method='POST',
                 renderer='kotti_site_settings:templates/options.pt')
    def process_update_settings(self):
        app_settings = AppSettings.query.filter(
            AppSettings.settings_id == self.context.id
        ).all()
        options = {
            aps.id: aps for aps in app_settings
        }
        for option, value in self.request.params.iteritems():
            if option in options:
                options[option].value = value
                DBSession.add(options[option])
        return httpexc.HTTPFound(location=self.context.path)

    @view_config(name='add-option', permission='admin',
                 request_method='GET',
                 renderer='kotti_site_settings:templates/add_option.pt')
    def add_option(self):
        return {}

    @view_config(name='add-option', permission='admin',
                 request_method='POST',
                 renderer='kotti_site_settings:templates/add_option.pt')
    def process_new_option(self):
        option = self.request.params.get("option", None)
        value = self.request.params.get("value", "")
        if option:
            setting = AppSettings(
                id=util.slugify(option),
                option=option,
                value=value,
                settings_id=self.context.id
            )
            DBSession.add(setting)
            self.request.session.flash(
                "Option has been added",
                "success"
            )
        else:
            self.request.session.flash(
                "No option was given",
                "danger"
            )
        return {}
